//sockerr.cpp

   /*-----------------------------------------------------------------------*\
   | OWLSock For Windows v1.0                                                |
   --------------------------------------------------------------------------|
   | Written By:  Paul Pedriana                                              |
   | Date:        May 7, 1995.                                               |
   | Copyright:   Copyright (c) 1995 by Paul Pedriana.  All Rights Reserved. |
   | UserID(s):   70541,3223                                                 |
   |              70541.3223@compuserve.com                                  |
   --------------------------------------------------------------------------|
   | OWLSock is a Winsock C++ class library for OWL and Windows (16 and 32). |
   | It makes the use of Winsock much simpler and organizes Winsock's        |
   | functionality into logical classes.  See owlsock.doc and the owlsock    |
   | source files for documentation on how to use OWLSock.                   |
   \*-----------------------------------------------------------------------*/

#include <owl\owlpch.h>
#pragma hdrstop

#include "sockerr.hpp"
#include <winsock.h>
#include <stdio.h>

extern HINSTANCE _hInstance;   //Defined by startup code for windows programs.

//-***********************************************************************
SocketError::SocketError(int nNewError, int nNewSizeToAllocate)
  :nError(nNewError),szString(NULL), nSizeToAllocate(nNewSizeToAllocate)
{
   GetErrorString();
}


SocketError::~SocketError(){
   if(szString)   //This 'if' statement not really needed in C++.
      delete[] szString;
}

SocketError& SocketError::operator=(SocketError& newSocketError){
   nError = newSocketError.nError;
   if(szString)   //This 'if' statement not really needed in C++.
      delete[] szString;
   szString = new char[strlen(newSocketError.szString)+1];
   strcpy(szString, newSocketError.szString);
   return *this;
}

int operator==(SocketError& socketError1, SocketError& socketError2){
//The important criteria for determining equality is the nError value.  The
//  string is unimportant.
   return (socketError1.nError == socketError2.nError);
}

void SocketError::ISocketError(int nNewError){
   nError = nNewError;
   GetErrorString();
}

int SocketError::GetReasonValue(){
   return nError;
}

char* SocketError::GetReasonString(){
//This function simply hands the pointer to the string to the caller.  The caller
//  should not mess with this string, as it doesn't belong to him.
   return szString;
}

char* SocketError::CopyReasonString(char* szDestinationString){
//This function makes a copy of the reasong string for the user.
//szString must be large enough to hold the string (N_MAX_REASON_STRING)
   strcpy(szDestinationString, szString);
   return szDestinationString;
}

char* SocketError::AppendError(char* szStringToAppendErrorTo, char* szDestination){
//This function appends an error string to whatever is in the string
//  'szStringToAppendErrorTo' and put the result in szDestination.  You may
//  want to put something specific about the error in the string and then use
//  AppendError() to add the Winsock error code and description to it.
//  For example, you could say:
//    MessageBox(SocketError(WSAENOTCONN).AppendError("Unable to send your mail"),
//                                                   "Error", MB_OK);
//  And AppendError() will put "\n\nWinsock Error 10057: Socket is not presently connected"
//    after the "Unable to send you mail" string.  Quite convenient.
//szStringToAppendErrorTo must be able to hold at least 128 characters.

   if(!szDestination){ //In this case, the user wants to use our own string.
      char* szTempString = new char[nSizeToAllocate];
      strcpy(szTempString, szStringToAppendErrorTo);
      strcat(szTempString, "\n\n");
      strcat(szTempString, GetReasonString());
      strcpy(szString, szTempString);
      delete[] szTempString;
      return szString;  //return our own string.
   }

   strcpy(szDestination, szStringToAppendErrorTo);
   strcat(szDestination, "\n\n");
   strcat(szDestination, GetReasonString());
   return szDestination;
}

char* SocketError::AppendError(int nStringResourceToAppendErrorTo, char* szDestination){
//This function is similar to AppendError(char*), but the pre-string is taken from
//  a string resource and 'szStringToAppendErrorTo' will be overwritten with what is
//  in the string resource and appended with the Winsock Error description.
//szStringToAppendErrorTo must be able to hold at least 128 characters.
   if(!szDestination){ //In this case, the user wants to use our own string.
      char* szTempString = new char[nSizeToAllocate];
      ::LoadString(_hInstance, nStringResourceToAppendErrorTo, szTempString, 90);
      AppendError(szTempString);
      strcpy(szString, szTempString);
      delete[] szTempString;
      return szString;
   }
   ::LoadString(_hInstance, nStringResourceToAppendErrorTo, szDestination, 90);
   return AppendError(szDestination);
}



void SocketError::GetErrorString(){
//This function gets a string, suitable for display, based on the nError value.
//The previous string is deleted if necessary.
//Note that the string allocated is ALWAYS at least 128 characters long.
//Note that even though the error strings you see below don't have
//  error numbers associated with them, the function pre-pends the error number
//  to the szString before returning.  If you are writing string resources
//  for the error strings, don't put error numbers in the string, since we
//  do that for you below.  
   char* szTemp;
   if(szString)
      delete[] szString;
   szString = new char[nSizeToAllocate];
   szString[0]=0;

#ifdef B_USING_STRING_RESOURCES
   #ifndef N_STRING_RESOURCE_BASE
      #define N_STRING_RESOURCE_BASE 0
   #endif
   //Use string resources from application resource file. It can be of the form:
   //  "(WSAEBLAH): Blah error incountered"
   //See below for static versions of these strings.
   char* szTemp1 = new char[nSizeToAllocate];
   ::LoadString(_hInstance, N_STRING_RESOURCE_BASE+nError, szTemp1, nSizeToAllocate);
   sprintf(szString, "Winsock Error %d %s", nError, szTemp1);
   delete[] szTemp1;
#else
   switch (nError){
      case WSAEINTR:
         szTemp = "(WSAEINTR): Interrupted system call"; break;
      case WSAEBADF:
         szTemp = "(WSAEBADF): Bad file number"; break;
      case WSAEACCES:
         szTemp = "(WSAEACCES): Permission denied"; break;
      case WSAEFAULT:
         szTemp = "(WSAEFAULT): Bad address passed"; break;
      case WSAEINVAL:
         szTemp = "(WSAEINVAL): Invalid parameter passed"; break;
      case WSAEMFILE:
         szTemp = "(WSAEMFILE): Too many open files"; break;
      case WSAEWOULDBLOCK:
         szTemp = "(WSAEWOULDBLOCK): Operation would block"; break;
      case WSAEINPROGRESS:
         szTemp = "(WSAEINPROGRESS): Operation is now in progress"; break;
      case WSAEALREADY:
         szTemp = "(WSAEALREADY): Operation is already in progress"; break;
      case WSAENOTSOCK:
         szTemp = "(WSAENOTSOCK): Socket operation on non-socket"; break;
      case WSAEDESTADDRREQ:
         szTemp = "(WSAEDESTADDRREQ): Destination address required"; break;
      case WSAEMSGSIZE:
         szTemp = "(WSAEMSGSIZE): Message is too long"; break;
      case WSAEPROTOTYPE:
         szTemp = "(WSAEPROTOTYPE): The protocol is of the wrong type for the socket"; break;
      case WSAENOPROTOOPT:
         szTemp = "(WSAENOPROTOOPT): The requested protocol is not available"; break;
      case WSAEPROTONOSUPPORT:
         szTemp = "(WSAEPROTONOSUPPORT): The requested protocol is not supported"; break;
      case WSAESOCKTNOSUPPORT:
         szTemp = "(WSAESOCKTNOSUPPORT): The specified socket type is not supported"; break;
      case WSAEOPNOTSUPP:
         szTemp = "(WSAEOPNOTSUPP): The specified operation is not supported"; break;
      case WSAEPFNOSUPPORT:
         szTemp = "(WSAEPFNOSUPPORT): The specified protocol family is not supported"; break;
      case WSAEAFNOSUPPORT:
         szTemp = "(WSAEAFNOSUPPORT): The specified address family is not supported"; break;
      case WSAEADDRINUSE:
         szTemp = "(WSAEADDRINUSE): The specified address is already in use"; break;
      case WSAEADDRNOTAVAIL:
         szTemp = "(WSAEADDRNOTAVAIL): The requested address is unassignable"; break;
      case WSAENETDOWN:
         szTemp = "(WSAENETDOWN): The network appears to be down"; break;
      case WSAENETUNREACH:
         szTemp = "(WSAENETUNREACH): The network is unreachable"; break;
      case WSAENETRESET:
         szTemp = "(WSAENETRESET): The network dropped the connection on reset"; break;
      case WSAECONNABORTED:
         szTemp = "(WSAECONNABORTED): Software caused a connection abort"; break;
      case WSAECONNRESET:
         szTemp = "(WSAECONNRESET): Connection was reset by peer"; break;
      case WSAENOBUFS:
         szTemp = "(WSAENOBUFS): Out of buffer space"; break;
      case WSAEISCONN:
         szTemp = "(WSAEISCONN): Socket is already connected"; break;
      case WSAENOTCONN:
         szTemp = "(WSAENOTCONN): Socket is not presently connected"; break;
      case WSAESHUTDOWN:
         szTemp = "(WSAESHUTDOWN): Can't send data because socket is shut down"; break;
      case WSAETOOMANYREFS:
         szTemp = "(WSAETOOMANYREFS): Too many references, unable to splice"; break;
      case WSAETIMEDOUT:
         szTemp = "(WSAETIMEDOUT): The connection timed out"; break;
      case WSAECONNREFUSED:
         szTemp = "(WSAECONNREFUSED): The connection was refused"; break;
      case WSAELOOP:
         szTemp = "(WSAELOOP): Too many symbolic link levels"; break;
      case WSAENAMETOOLONG:
         szTemp = "(WSAENAMETOOLONG): File name is too long"; break;
      case WSAEHOSTDOWN:
         szTemp = "(WSAEHOSTDOWN): The host appears to be down"; break;
      case WSAEHOSTUNREACH:
         szTemp = "(WSAEHOSTUNREACH): The host is unreachable"; break;
      case WSAENOTEMPTY:
         szTemp = "(WSAENOTEMPTY): The directory is not empty"; break;
      case WSAEPROCLIM:
         szTemp = "(WSAEPROCLIM): There are too many processes"; break;
      case WSAEUSERS:
         szTemp = "(WSAEUSERS): There are too many users"; break;
      case WSAEDQUOT:
         szTemp = "(WSAEDQUOT): The disk quota is exceeded"; break;
      case WSAESTALE:
         szTemp = "(WSAESTALE): Bad NFS file handle"; break;
      case WSAEREMOTE:
         szTemp = "(WSAEREMOTE): There are too many levels of remote in the path"; break;
      case WSAEDISCON:
         szTemp = "(WSAEDISCO): Disconnect"; break;
      case WSASYSNOTREADY:
         szTemp = "(WSASYSNOTREADY): Network sub-system is not ready or unusable"; break;
      case WSAVERNOTSUPPORTED:
         szTemp = "(WSAVERNOTSUPPORTED): The requested version is not supported"; break;
      case WSANOTINITIALISED:
         szTemp = "(WSANOTINITIALISED): Socket system is not initialized"; break;
      case WSAHOST_NOT_FOUND:
         szTemp = "(WSAHOST_NOT_FOUND): The host was not found"; break;
      case WSATRY_AGAIN:
         szTemp = "(WSATRY_AGAIN): Non-authoritative host not found"; break;
      case WSANO_RECOVERY:
         szTemp = "(WSANO_RECOVERY): Non-recoverable error"; break;
      case WSANO_DATA:
         szTemp = "(WSANO_DATA): No information of the requested type was available"; break;
      default:
         szTemp = "(???): Unknown error or no error";
   }
   sprintf(szString, "Winsock Error %d %s", nError, szTemp);
#endif
}





