//sockets.hpp

   /*-----------------------------------------------------------------------*\
   | OWLSock For Windows v1.0                                                |
   --------------------------------------------------------------------------|
   | Written By:  Paul Pedriana                                              |
   | Date:        May 7, 1995.                                               |
   | Copyright:   Copyright (c) 1995 by Paul Pedriana.  All Rights Reserved. |
   | UserID(s):   70541,3223                                                 |
   |              70541.3223@compuserve.com                                  |
   --------------------------------------------------------------------------|
   | OWLSock is a Winsock C++ class library for OWL and Windows (16 and 32). |
   | It makes the use of Winsock much simpler and organizes Winsock's        |
   | functionality into logical classes.  See owlsock.doc and the owlsock    |
   | source files for documentation on how to use OWLSock.                   |
   \*-----------------------------------------------------------------------*/

#ifndef _SOCKETS_HPP
#define _SOCKETS_HPP

#include <owl\owlpch.h>
#pragma hdrstop

#include <winsock.h>
#include "address.hpp"

#define WINSOCK_NOERROR (int)0
#define WINSOCK_ERROR   (int)SOCKET_ERROR

#define N_DEF_MAX_READ_BUFFFER_SIZE 8192

//Define this somewhere before compiling OWLSock to enable Data Queues (internal queueing).
//#define B_USE_DATA_QUEUES

//-********************************************************************
#ifdef B_USE_DATA_QUEUES
class QueueItem{
public:
   static unsigned long lLastSerialNumber;
   unsigned long        lSerialNumber;     //This is defined as being unique for all QueueItems.
   char*                chData;
   int                  nDataLength;
   int                  nPresentPosition;
   short                bShouldDelete;
   SocketAddress        sAddress;          //The send or receive address associated with the queue item.

   QueueItem();
   QueueItem(QueueItem& newQueueItem);
   QueueItem(int nLength, short bNewShouldDelete=1); //For newly created QueueItems with no data ready for them yet.
   QueueItem(SocketAddress& sNewAddress, char* chData, int nLength, short bNewShouldDelete=1, short bCopyData=1);
   virtual ~QueueItem();
   QueueItem& operator=(const QueueItem& newQueueItem);
   friend int operator==(const QueueItem& QueueItem1, const QueueItem& QueueItem2);
   friend int operator!=(const QueueItem& QueueItem1, const QueueItem& QueueItem2);
   void CopyQueueItem(const QueueItem& sourceQueueItem);
   void SetAddress(SocketAddress& sNewAddress);
   void ClearData();
   void SetSerialNumber(unsigned long lNewSerialNumber=++lLastSerialNumber){lSerialNumber=lLastSerialNumber;}
};
#endif //B_USE_DATA_QUEUES


//-********************************************************************
#define MSG_SOCKET_NOTIFY ((UINT)(WM_USER+301))
#define FD_ALL            (FD_READ|FD_WRITE|FD_OOB|FD_ACCEPT|FD_CONNECT|FD_CLOSE)
#define FD_NONE           (0)

//These enumerations duplicate those defined in 'winsock.h'
//You can OR or add these together to make a notification set.
enum NotificationSet{nNotifyNone  = 0x00, nNotifyRead   = 0x01, nNotifyWrite   = 0x02,
                     nNotifyOOB   = 0x04, nNotifyAccept = 0x08, nNotifyConnect = 0x10,
                     nNotifyClose = 0x20, nNotifyAll    = 0x3F};
class Socket;

class SocketWindow :public TWindow{
public:
   static UINT snMsgSocketNotification; //Message used to notify hwndNotification, it hwndNotification is used.
   Socket*     mySocketParent;          //Exported so Parent can have easy access to it.

   SocketWindow(Socket* newSocketParent);
   SocketWindow& operator =(SocketWindow& newSocketWindow);

   void SetNotificationSet(int nNewNotificationSet){nNotificationSet=nNewNotificationSet;}
   void SetNotificationWindow(TWindow* windowNewNotification){windowNotification = windowNewNotification;}
   int GetLastError(){return nLastError;}
   int StartAcceptNotification();
   int StartRegularNotification();
   int StartCustomNotification(int nNewSelectOptions);
   int CancelNotification();

protected:
   int             nSelectOptions;    //We need to keep our own copy of this so we can do an assignment operator.
   TWindow*        windowNotification;   //A second window that can be notified instead of the Socket.
   int             nNotificationSet;
   int             nLastError;
   LRESULT DoNotification(WPARAM wParam, LPARAM lParam);
   DECLARE_RESPONSE_TABLE(SocketWindow);
};


//**************************************************************************************
#ifdef B_USE_DATA_QUEUES
#include <classlib\queues.h>
typedef TIQueueAsVector<QueueItem> DataQueue;
typedef TIQueueAsVectorIterator<QueueItem> QueueItemIterator;
#endif //B_USE_DATA_QUEUES

class Socket{
public:
   enum ShutMode{nShutModeNoRecv = 0, nShutModeNoSend = 1, nShutModeNoRecvSend = 2};

   SOCKET         s;                   //The Socket handle
   short          bBound;              //A flag that we can use to tell if socket is bound or not.  It gets set on automatically, but doesn't get set off automatically.
   SocketAddress  mySocketAddress;     //My address.
   SocketAddress  myPeerSocketAddress; //Address of Peer to communicate with.  Used for datagrams and connections.

   Socket();
   Socket(SOCKET& newS);
   Socket(SocketAddress& newSocketAddress, int nNewFamily=PF_INET, int nNewType=SOCK_STREAM, int nNewProtocol=0);
   virtual ~Socket();
   operator SOCKET() const {return s;}
   virtual int CloseSocket();
   virtual int ShutDownSocket(int nShutMode = nShutModeNoRecvSend);
   Socket& operator=(Socket& newSocket);
   friend int operator==(const Socket& socket1, const Socket& socket2);

   //Commands
   virtual void SetNotificationSet(int nNewNotificationSet){myWindow.SetNotificationSet(nNewNotificationSet);}
   virtual void SetNotificationWindow(TWindow* windowNewNotification){myWindow.SetNotificationWindow(windowNewNotification);}
   virtual int  CreateSocket();
   virtual int  BindSocket(SocketAddress& addressToBindTo); //binds to a given address.
   virtual int  BindSocket();                                  //binds to our address.
   virtual void SetMyAddress(SocketAddress& newSocketAddress);
   virtual void SetPeerSocketAddress(SocketAddress& newPeerSocketAddress);
   virtual void SetSocketStyle(int nNewFamily=PF_INET, int nNewType=SOCK_STREAM, int nNewProtocol=0);
   virtual int StartAcceptNotification();
   virtual int StartRegularNotification();
   virtual int StartCustomNotification(int nSelectionOptions);
   virtual int CancelNotification();
   short SetSaveSocketOnDelete(short bNewSaveSocket=TRUE){return bSaveSocket = bNewSaveSocket;} //Tells class Sockete not to close the socket on deletion.
   virtual int  ConvertProtocol(char* szNewProtocol);
   #ifdef B_USE_DATA_QUEUES
   virtual void CopyQueue(DataQueue& queueDestination, DataQueue& queueSource);
   #endif

   //Info
   int GetLastError(){return nLastError;}
   virtual unsigned long GetDriverWaitingSize();
   virtual unsigned long GetTotalWaitingSize();
   #ifdef B_USE_DATA_QUEUES
   virtual unsigned long GetLocalWaitingSize();
   virtual unsigned long GetLocalWaitingSizeQueue(DataQueue& queue);
   virtual int GetLocalWaitingCount();     //Counts total number of QueueItems in the in-queue
   #endif //B_USE_DATA_QUEUES
   virtual int GetMyAddress(SocketAddress& socketAddress, int& nAddressLength, SOCKET& socket);
   virtual int GetMyAddress(SocketAddress& socketAddress, int& nAddressLength);
   virtual int GetPeerAddress(SocketAddress& socketAddress, int& nAddressLength, SOCKET& socket);
   virtual int GetPeerAddress(SocketAddress& socketAddress, int& nAddressLength);

   //Read/Write (Send/Receive)
   virtual void SetMaxReadBufferSize(int nNewMaxReadBufferSize);
   #ifdef B_USE_DATA_QUEUES
   virtual int PeekQueue(DataQueue& queue, char* chData, int& nBytesToRead, SocketAddress& sAddress, int nBuffersToRead);
   virtual int Peek(char* chData, int& nBytesToRead, SocketAddress& sAddress, int nBuffersToRead);
   #endif

   //Options:
   int SetBroadcastOption(BOOL bBroadcast);
   int SetDebugOption(BOOL bDebug);
   int SetLingerOption(BOOL bLinger, u_short nLingerTime=0);
   int SetRouteOption(BOOL bRoute);
   int SetKeepAliveOption(BOOL bKeepAlive);
   int SetOOBOption(BOOL bSendOOBDataInline);
   int SetReceiveBufferOption(int nReceiveBufferSize);
   int SetSendBufferOption(int nSendBufferSize);
   int SetReuseAddressOption(BOOL bAllowReuseAddress);

   int GetBroadcastOption(BOOL& bBroadcast);
   int GetDebugOption(BOOL& bDebug);
   int GetLingerOption(BOOL& bLinger, u_short& nLingerTime);
   int GetRouteOption(BOOL& bRoute);
   int GetKeepAliveOption(BOOL& bKeepAlive);
   int GetOOBOption(BOOL& bSendOOBDataInline);
   int GetReceiveBufferOption(int& nReceiveBufferSize);
   int GetSendBufferOption(int& nSendBufferSize);
   int GetReuseAddressOption(BOOL& bAllowReuseAddress);

protected:
   int            nFamily;             //PF_INET, etc. (this is the protocol family)
   int            nType;               //SOCK_STREAM, etc.
   int            nProtocol;           //IPPROTO_TCP, etc.
   int            nLastError;          //myLast Error.
   int            nMaxReadBufferSize;  //
   short          bSaveSocket;         //Save the socket on deletion?
   SocketWindow   myWindow;            //Will receive internal notifications and pass them to this class.
   friend class   SocketWindow;
   #ifdef B_USE_DATA_QUEUES
   DataQueue      myIncomingQueue;     //Data that has been already received.
   DataQueue      myOutgoingQueue;     //Data to be sent.
   #endif

   //protected initialization.
   void ISocket();      //Sets up the friend window.

   //Notification:
   virtual DoReadNotification(const SOCKET& s, int nError);
   virtual DoWriteNotification(const SOCKET& s, int nError);
   virtual DoOOBNotification(const SOCKET& s, int nError);
   virtual DoAcceptNotification(const SOCKET& s, int nError);
   virtual DoConnectNotification(const SOCKET& s, int nError);
   virtual DoCloseNotification(const SOCKET& s, int nError);
};

#endif //_SOCKETS_HPP



