//socksub.hpp

   /*-----------------------------------------------------------------------*\
   | OWLSock For Windows v1.0                                                |
   --------------------------------------------------------------------------|
   | Written By:  Paul Pedriana                                              |
   | Date:        May 7, 1995.                                               |
   | Copyright:   Copyright (c) 1995 by Paul Pedriana.  All Rights Reserved. |
   | UserID(s):   70541,3223                                                 |
   |              70541.3223@compuserve.com                                  |
   --------------------------------------------------------------------------|
   | OWLSock is a Winsock C++ class library for OWL and Windows (16 and 32). |
   | It makes the use of Winsock much simpler and organizes Winsock's        |
   | functionality into logical classes.  See owlsock.doc and the owlsock    |
   | source files for documentation on how to use OWLSock.                   |
   \*-----------------------------------------------------------------------*/

#ifndef _SOCK_SUB
#define _SOCK_SUB

#include <owl\owlpch.h>
#pragma hdrstop

#include "sockets.hpp"

class DatagramSocket: public Socket{
public:
   DatagramSocket();
   DatagramSocket(SOCKET& newS);
   DatagramSocket(SocketAddress& newSocketAddress, int nNewAddressFormat=PF_INET, int nNewType=SOCK_DGRAM, int nNewProtocol=0);
   DatagramSocket& operator=(DatagramSocket& newDatagramSocket1)
                             {Socket::operator=(newDatagramSocket1); return *this;}
   void SetMaxPacketSendSize(int nNewSize){nMaxPacketSendSize = nNewSize;}
   #ifdef B_USE_DATA_QUEUES
   int PeekNextSender(SocketAddress& sAddress);
   #endif //B_USE_DATA_QUEUES
   int Read(char* chData, int& nCharsToRead, SocketAddress& sAddress);
   int Write(char* chData, int& nCharsToWrite, SocketAddress& theOutSocketAddress, short bBecomeOwnerOfData=1, short bCopyData=1);
   int Write(char* chData, int& nCharsToWrite, short bBecomeOwnerOfData=1, short bCopyData=1);

protected:
   static int nMaxPacketSendSize;  //Defined by the WSAStartup() call return information (WSAData).

   DoReadNotification(const SOCKET& s, int nError);
   DoWriteNotification(const SOCKET& s, int nError);
   #ifdef B_USE_DATA_QUEUES
   int AttemptReceiveQueue(DataQueue& queue, int nFlags=0); //Reads as much data as possible (until a block) from the given queue.
   int AttemptSendQueue(DataQueue& queue, int nFlags=0);
   #endif //B_USE_DATA_QUEUES
};


#define N_DEF_MAX_QUEUED_CONNECTIONS 5
class StreamSocket : public Socket{
public:
   enum ConnectStatus{nNotConnected, nConnectPending, nConnected, nListening} nConnectStatus;

   StreamSocket();
   StreamSocket(SOCKET& newS);
   StreamSocket(SocketAddress& newSocketAddress, int nNewAddressFormat=PF_INET, int nNewType=SOCK_STREAM, int nNewProtocol=0);
   StreamSocket& operator=(StreamSocket& newStreamSocket);

   Listen(int nMaxQueuedConnections=N_DEF_MAX_QUEUED_CONNECTIONS);
   Connect();
   Connect(SocketAddress& addressToConnectTo);
   Accept(StreamSocket& socket);
   Accept(SOCKET& socket, sockaddr& sAddress);
   int Read(char* chData, int& nCharsToRead);
   int Write(char* chData, int& nCharsToWrite, int nFlags=0, short bBecomeOwnerOfData=1, short bCopyData=1); //Is the same as send func
   int ReadOOB(char* chData, int& nCharsToRead);
   int WriteOOB(char* chData, int& nCharsToWrite, int nFlags=MSG_OOB, short bBecomeOwnerOfData=1, short bCopyData=1); //ORs nFlags w MSG_OOB.
   #ifdef B_USE_DATA_QUEUES
   int GetLocalWaitingSizeOOB();
   int GetLocalWaitingCountOOB();
   int ReadQueue(DataQueue& queue, char* chData, int& nCharsToRead);
   int WriteQueue(DataQueue& queue, char* chData, int& nCharsToWrite, int nFlags=0, short bBecomeOwnerOfData=1, short bCopyData=1); //Is the same as send func
   int PeekOOB(char* chData, int& nCharsToRead);
   #endif //B_USE_DATA_QUEUES

protected:
   #ifdef B_USE_DATA_QUEUES
   DataQueue     myIncomingOOBQueue;    //OOB data that has been already received.
   DataQueue     myOutgoingOOBQueue;    //OOB data to be sent.
   #endif //B_USE_DATA_QUEUES

   #ifdef B_USE_DATA_QUEUES
   int AttemptReceiveQueue(DataQueue& queue, int nFlags=0); //Reads as much data as possible (until a block) from the given queue.
   int AttemptSendQueue(DataQueue& queue, int nFlags=0);
   #endif //B_USE_DATA_QUEUES

   DoReadNotification(const SOCKET& s, int nError);
   DoWriteNotification(const SOCKET& s, int nError);
   DoOOBNotification(const SOCKET& s, int nError);
   DoAcceptNotification(const SOCKET& s, int nError);
   DoConnectNotification(const SOCKET& s, int nError);
   DoCloseNotification(const SOCKET& s, int nError);
};

#endif //_SOCK_SUB


